import socket
import threading
import json
import time

SERVER = 'localhost'
PORT = 12345

username = None
client = socket.socket()


def listen_for_messages():
    while True:
        try:
            data = client.recv(4096).decode()
            if not data:
                break
            msg = json.loads(data)
            if msg['type'] == 'message':
                print(f"\n📩 Nouveau message de {msg['from']} : {msg['text']}")
                print(">>> ", end='', flush=True)
        except:
            break


def auth_menu():
    global username
    while True:
        print("=== Authentification ===")
        print("1. Connexion")
        print("2. Inscription")
        choice = input("> ")

        if choice not in ["1", "2"]:
            print("Choix invalide.")
            continue

        username = input("Nom d’utilisateur : ").strip()
        password = input("Mot de passe : ").strip()

        action = "login" if choice == "1" else "signup"

        msg = {
            "type": "auth",
            "action": action,
            "username": username,
            "password": password
        }

        client.send(json.dumps(msg).encode())
        response = json.loads(client.recv(4096).decode())

        if response['status'] == 'ok':
            print("✅ Connexion réussie.")
            return
        elif response['status'] == 'fail':
            print("❌ Nom d’utilisateur ou mot de passe incorrect.")
        elif response['status'] == 'exists':
            print("⚠️ Utilisateur déjà existant.")


def main_menu():
    amis = []

    while True:
        print("\n=== MiniChat ===")
        print("1. Gérer mes amis")
        print("2. Envoyer un message")
        print("3. Quitter")
        choice = input("> ")

        if choice == "1":
            print("\nAmis actuels :")
            for i, a in enumerate(amis):
                print(f"{i+1}. {a}")
            print("a. Ajouter un ami")
            print("s. Supprimer un ami")
            sub = input("> ")

            if sub == "a":
                nom = input("Nom de l’ami : ").strip()
                if nom and nom not in amis:
                    amis.append(nom)
            elif sub == "s":
                idx = input("Numéro à supprimer : ")
                if idx.isdigit() and 1 <= int(idx) <= len(amis):
                    amis.pop(int(idx) - 1)

        elif choice == "2":
            if not amis:
                print("Aucun ami.")
                continue
            print("Choisir un ami :")
            for i, a in enumerate(amis):
                print(f"{i+1}. {a}")
            idx = input("> ")
            if not idx.isdigit() or not (1 <= int(idx) <= len(amis)):
                print("Choix invalide.")
                continue
            ami = amis[int(idx) - 1]
            texte = input(">>> ").strip()
            if texte:
                msg = {
                    "type": "message",
                    "to": ami,
                    "from": username,
                    "text": texte,
                    "timestamp": time.strftime("%Y-%m-%d %H:%M:%S")
                }
                client.send(json.dumps(msg).encode())
                print("Message envoyé.")

        elif choice == "3":
            print("Déconnexion...")
            break
        else:
            print("Choix invalide.")


def start_client():
    try:
        client.connect((SERVER, PORT))
        auth_menu()
        threading.Thread(target=listen_for_messages, daemon=True).start()
        main_menu()
    except Exception as e:
        print(f"Erreur de connexion : {e}")
    finally:
        client.close()


if __name__ == "__main__":
    start_client()
